package de.osci.helper;

import javax.xml.XMLConstants;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

import de.osci.osci12.common.Constants;
import de.osci.osci12.common.Constants.CommonTags;
import de.osci.osci12.common.Constants.HeaderTags;


/**
 * <p>
 * Copyright  2017 Freie Hansestadt Bremen
 * </p>
 * <p>
 * Erstellt von Governikus GmbH & Co.  KG
 * </p>
 * <p>
 * Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr freie Softwarebibliotheken genutzt
 * werden.
 * </p>
 * Die Lizenzbestimmungen knnen unter der URL <a href='www.osci.de' >www.osci.de</a > abgerufen oder bei der Senator fr
 * Finanzen, Freie Hansestadt Bremen,  Schillerstr. 1, 28195 Bremen schriftlich angefordert werden.
 * </p>
 * @author R. Lindemann
 * @version 1.8.0
 */
public class ParserHelper
{

  private static Log log = LogFactory.getLog(ParserHelper.class);

  public static void setFeatures(XMLReader reader)
  {
    // set to false
    addFeature(reader, "http://xml.org/sax/features/external-general-entities", false);
    addFeature(reader, "http://xml.org/sax/features/external-parameter-entities", false);
    addFeature(reader, "http://xml.org/sax/features/validation", false);

    // set to true
    addFeature(reader, "http://xml.org/sax/features/namespaces", true);
    addFeature(reader, "http://xml.org/sax/features/namespace-prefixes", true);

    // neue Schalter
    addFeature(reader, "http://apache.org/xml/features/disallow-doctype-decl", true);
    addFeature(reader, "http://apache.org/xml/features/nonvalidating/load-external-dtd", false);
    addFeature(reader, "http://apache.org/xml/features/nonvalidating/load-dtd-grammar", false);
    addFeature(reader, XMLConstants.FEATURE_SECURE_PROCESSING, true);
  }


  private static void addFeature(XMLReader reader, String feature, boolean value)
  {
    try
    {
      reader.setFeature(feature, value);

    }
    catch (SAXException e)
    {
      if (log.isDebugEnabled())
        log.debug("warning: Parser does not support feature (" + feature + ")");
    }
  }


  /**
   * Prft, ob der bergebene symmetrische Verschlsselungsalgorithmus blockiert ist (siehe
   * {@link Constants}). Man kann die System Property 'gov.osciserver.blockCBC=true' nutzen, um CBC-basierte
   * Algorithmen zu blockieren.
   * 
   * @param symmetricAlgorithm
   * @return
   */
  public static boolean isSymmetricAlgorithmBlocked(String symmetricAlgorithm)
  {
    if (symmetricAlgorithm == null)
    {
      return true;
    }

    // block cbc algo if system property 'gov.osciserver.blockCBC' is set to true
    if (symmetricAlgorithm.endsWith("-cbc"))
    {
      if ("true".equals(System.getProperty("de.osci.GCMAlgorithmOnly")))
      {
        return true;
      }
      else
      {
        return false;
      }
    }
    if (symmetricAlgorithm.endsWith("-gcm"))
    {
      return false;
    }
    return true;
  }

  /**
   * Prft ob die Secure Content parsing angestellt ist. Hierbei werden die Content RefIds auf doppelte
   * Eintrge berprft.
   * 
   * @return
   */
  public static boolean isSecureContentDataCheck()
  {
    if ("false".equals(System.getProperty("de.osci.SecureContentDataCheck")))
    {
      return false;
    }
    return true;
  }

  /**
   * Prft ob die Secure Transport parsing angestellt ist. Hierbei werden die smtlich RefIds auf
   * Transportebene auf doppelte Eintrge berprft.
   * 
   * @return
   */
  public static boolean isSecureTransportDataCheck()
  {
    if ("false".equals(System.getProperty("de.osci.SecureTransportDataCheck")))
    {
      return false;
    }
    return true;
  }

  /**
   * Prft ob nur ein symmetrischer Verschlsselungsalgorithmus mit GCM-Modus erlaubt sein soll.
   * 
   * @return
   */
  public static boolean isGCMAlgorithmOnly()
  {
    if ("true".equals(System.getProperty("de.osci.GCMAlgorithmOnly")))
    {
      return true;
    }
    return false;
  }

  public static boolean isElement(HeaderTags expectedElement, String elementName, String uri)
  {
    if (expectedElement.getElementName().equals(elementName)
        && expectedElement.getNamespace().getUri().equals(uri))
    {
      return true;
    }else
    {
      return false;
    }
  }
  public static boolean isElement(CommonTags expectedElement, String elementName, String uri)
  {
    if (expectedElement.getElementName().equals(elementName)
        && expectedElement.getNamespace().getUri().equals(uri))
    {
      return true;
    }else
    {
      return false;
    }
  }
}
