package de.osci.osci12.common;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Hashtable;
import java.util.Map;
import java.util.TimeZone;


/**
 * In dieser Klasse sind Konstanten der Bibliothek definiert.
 * <p>
 * Copyright  2017 Freie Hansestadt Bremen
 * </p>
 * <p>
 * 
 * </p>
 * <p>
 * Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr freie Softwarebibliotheken genutzt
 * werden.
 * </p>
 * Die Lizenzbestimmungen knnen unter der URL <a href='<a href='www.osci.de' >www.osci.de</a >'>www.osci.de</a> abgerufen oder bei der
 * Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1, 28195 Bremen schriftlich angefordert werden.
 * </p>
 *
 * @author R. Lindemann, N.Bngener, J. Wothe
 * @version 1.8.0
 */
public class Constants
{

  /**
   * Enumeration fr neue OSCI Features.
   * 
   * @author Lindemann
   */
  public enum OSCIFeatures
  {
    DisableBase64("1.2.0"),
    OAEPEncryption("1.6.0"),
    SHA3HashAlgo("1.6.3"),
    GCMPaddingModus("1.7.0"),
    PartialMessageTransmission("1.8.0");

    private String version;

    OSCIFeatures(String version)
    {
      this.version = version;
    }

    /**
     * @return Version der OSCI Bibliothek ab der dieses Feature untersttzt wird.
     */
    public String getVersion()
    {
      return version;
    }
  }

  public enum CommonTags
  {
    ContentPackage("ContentPackage", Namespaces.OSCI),
    ProcessCardBundle("ProcessCardBundle", Namespaces.OSCI),
    ReplyProcessCardBundle("ReplyProcessCardBundle", Namespaces.OSCI),
    RequestProcessCardBundle("RequestProcessCardBundle", Namespaces.OSCI),
    Feedback("Feedback", Namespaces.OSCI),
    MessageId("MessageId", Namespaces.OSCI),
    ChunkInformation("ChunkInformation", Namespaces.OSCI2017),
    InsideFeedback("InsideFeedback", Namespaces.OSCI2017);

    private String elementName;

    private Namespaces namespace;

    CommonTags(String elementName, Namespaces namespace)
    {
      this.elementName = elementName;
      this.namespace = namespace;
    }

    public String getElementName()
    {
      return elementName;
    }

    public Namespaces getNamespace()
    {
      return namespace;
    }
  }

  public enum HeaderTags
  {

    ControlBlock("ControlBlock", Namespaces.OSCI),
    ClientSignature("ClientSignature", Namespaces.OSCI),
    SupplierSignature("SupplierSignature", Namespaces.OSCI),
    DesiredLanguages("DesiredLanguages", Namespaces.OSCI),
    IntermediaryCertificates("IntermediaryCertificates", Namespaces.OSCI),
    QualityOfTimestamp("QualityOfTimestamp", Namespaces.OSCI),
    NonIntermediaryCertificates("NonIntermediaryCertificates", Namespaces.OSCI),
    // msg Types
    acceptDelivery("acceptDelivery", Namespaces.OSCI),
    fetchDelivery("fetchDelivery", Namespaces.OSCI),
    processDelivery("processDelivery", Namespaces.OSCI),
    responseToExitDialog("responseToExitDialog", Namespaces.OSCI),
    responseToFetchDelivery("responseToFetchDelivery", Namespaces.OSCI),
    mediateDelivery("mediateDelivery", Namespaces.OSCI),
    responseToMediateDelivery("responseToMediateDelivery", Namespaces.OSCI),
    storeDelivery("storeDelivery", Namespaces.OSCI),
    responseToStoreDelivery("responseToStoreDelivery", Namespaces.OSCI),
    fetchProcessCard("fetchProcessCard", Namespaces.OSCI),
    responseToFetchProcessCard("responseToFetchProcessCard", Namespaces.OSCI),
    forwardDelivery("forwardDelivery", Namespaces.OSCI),
    responseToForwardDelivery("responseToForwardDelivery", Namespaces.OSCI),
    responseToAcceptDelivery("responseToAcceptDelivery", Namespaces.OSCI),
    responseToProcessDelivery("responseToProcessDelivery", Namespaces.OSCI),


    FeatureDescription("FeatureDescription", Namespaces.OSCI2017),
    partialStoreDelivery("partialStoreDelivery", Namespaces.OSCI2017),
    responseToPartialStoreDelivery("responseToPartialStoreDelivery", Namespaces.OSCI2017),
    partialFetchDelivery("partialFetchDelivery", Namespaces.OSCI2017),
    responseToPartialFetchDelivery("responseToPartialFetchDelivery", Namespaces.OSCI2017);

    private String elementName;

    private Namespaces namespace;

    HeaderTags(String elementName, Namespaces namespace)
    {
      this.elementName = elementName;
      this.namespace = namespace;
    }

    public String getElementName()
    {
      return elementName;
    }

    public Namespaces getNamespace()
    {
      return namespace;
    }
  }

  public enum Namespaces
  {
    SOAP("http://schemas.xmlsoap.org/soap/envelope/", "soap"),
    OSCI("http://<a href='www.osci.de' >www.osci.de</a >/2002/04/osci", "osci"),
    OSCI2017("http://xoev.de/transport/osci12/7", "osci2017"),
    XML_SIG("http://www.w3.org/2000/09/xmldsig#", "ds"),
    XML_ENC("http://www.w3.org/2001/04/xmlenc#", "xenc"),
    XML_SCHEMA("http://www.w3.org/2001/XMLSchema-instance", "xsi");

    private String uri;

    private String prefix;

    Namespaces(String uri, String prefix)
    {
      this.uri = uri;
      this.prefix = prefix;
    }

    public String getUri()
    {
      return uri;
    }

    public String getPrefix()
    {
      return prefix;
    }
  }
  
  public enum LanguageTextEntries
  {
     missing_entry
  }

  /**
   * Hashtable mit den bentigten JCA/JCE-Identifiern (not modifiable)
   */
  public static final Hashtable<String, String> JCA_JCE_MAP;

  /**
   * Character-Encoding UTF-8
   */
  public static final String CHAR_ENCODING = "UTF-8";

  /**
   * Algorithmus fr Zufallszahlenerstellung wird auf SHA1 Pseudo Random Number Generation gestellt,
   * JCA/JCE-Identifier
   */
  public static final String SECURE_RANDOM_ALGORITHM_SHA1 = "SHA1PRNG";

  /**
   * Algorithmus fr Signaturerstellung, Konstante fr SHA1withRSA
   */
  public static final String SIGNATURE_ALGORITHM_RSA_SHA1 = "http://www.w3.org/2000/09/xmldsig#rsa-sha1";

  /**
   * Algorithmus fr Signaturerstellung, Konstante fr SHA1withRSA, JCA/JCE-Identifier
   */
  private static final String SIGNATURE_ALGORITHM_RSA_SHA1_JCA_JCE = "SHA1withRSA";

  /**
   * Hashtable mit Ablaufdaten der Algorithmen und Schlssellngen (not modifiable)
   */
  public static final Hashtable<String, Date> OUT_DATES;

  /**
   * Ablaufdatum fr 1024 Bit RSA-Schlssellnge (AlgKat2008)
   */
  public static final Date OUT_DATE_KEYSIZE_1024;

  /**
   * Default-Gltigkeitsdatum fr Algorithmus-Prfungen
   */
  public static final Date ACTUAL_DATE;



  /**
   * Algorithmus fr Signaturerstellung, Konstante fr SHA256withRSA
   */
  public static final String SIGNATURE_ALGORITHM_RSA_SHA256 = "http://www.w3.org/2001/04/xmldsig-more#rsa-sha256";

  /**
   * Algorithmus fr Signaturerstellung, Konstante fr SHA256withRSA, JCA/JCE-Identifier
   */
  private static final String SIGNATURE_ALGORITHM_RSA_SHA256_JCA_JCE = "SHA256withRSA";


  /**
   * Algorithmus fr Signaturerstellung, Konstante fr SHA512withRSA
   */
  public static final String SIGNATURE_ALGORITHM_RSA_SHA512 = "http://www.w3.org/2001/04/xmldsig-more#rsa-sha512";

  /**
   * Algorithmus fr Signaturerstellung, Konstante fr SHA512withRSA, JCA/JCE-Identifier
   */
  private static final String SIGNATURE_ALGORITHM_RSA_SHA512_JCA_JCE = "SHA512withRSA";

  /**
   * Algorithmus fr Signaturerstellung, Konstante fr SHA256withRSAandMGF1 (PSS)
   */
  public static final String SIGNATURE_ALGORITHM_RSA_SHA256_PSS = "http://www.w3.org/2007/05/xmldsig-more#sha256-rsa-MGF1";

  /**
   * Algorithmus fr Signaturerstellung, Konstante fr SHA256withRSAandMGF1 (PSS), JCA/JCE-Identifier
   */
  private static final String SIGNATURE_ALGORITHM_RSA_SHA256_PSS_JCA_JCE = "SHA256withRSAandMGF1";

  /**
   * Algorithmus fr Signaturerstellung, Konstante fr SHA512withRSAandMGF1 (PSS)
   */
  public static final String SIGNATURE_ALGORITHM_RSA_SHA512_PSS = "http://www.w3.org/2007/05/xmldsig-more#sha512-rsa-MGF1";

  /**
   * Algorithmus fr Signaturerstellung, Konstante fr SHA512withRSAandMGF1 (PSS), JCA/JCE-Identifier
   */
  private static final String SIGNATURE_ALGORITHM_RSA_SHA512_PSS_JCA_JCE = "SHA512withRSAandMGF1";

  /**
   * Algorithmus fr Signaturerstellung, Konstante fr RIPEMD160withRSA
   */
  public static final String SIGNATURE_ALGORITHM_RSA_RIPEMD160_RFC4051 = "http://www.w3.org/2001/04/xmldsig-more/rsa-ripemd160";

  /**
   * Konstante fr Abwrtskompatibilitt nach Korrektur der RIPEMD160withRSA-Konstante (Version 1.5, Typo in
   * RFC 4051)
   */
  public static final String SIGNATURE_ALGORITHM_RSA_RIPEMD160 = "http://www.w3.org/2001/04/xmldsig-more#rsa-ripemd160";

  /**
   * Algorithmus fr Signaturerstellung, Konstante fr RIPEMD160withRSA, JCA/JCE-Identifier
   */
  private static final String SIGNATURE_ALGORITHM_RSA_RIPEMD160_JCA_JCE = "RIPEMD160withRSA";

  /**
   * Algorithmus fr Signaturerstellung, Konstante fr SHA256withECDSA
   */
  public static final String SIGNATURE_ALGORITHM_ECDSA_SHA256 = "http://www.w3.org/2001/04/xmldsig-more#ecdsa-sha256";

  /**
   * Algorithmus fr Signaturerstellung, Konstante fr SHA256withECDSA, JCA/JCE-Identifier
   */
  private static final String SIGNATURE_ALGORITHM_ECDSA_SHA256_JCA_JCE = "SHA256withCVC-ECDSA";

  /**
   * Algorithmus fr Signaturerstellung, Konstante fr SHA512withECDSA
   */
  public static final String SIGNATURE_ALGORITHM_ECDSA_SHA512 = "http://www.w3.org/2001/04/xmldsig-more#ecdsa-sha512";

  /**
   * Algorithmus fr Signaturerstellung, Konstante fr SHA512withECDSA, JCA/JCE-Identifier
   */
  private static final String SIGNATURE_ALGORITHM_ECDSA_SHA512_JCA_JCE = "SHA512withCVC-ECDSA";

  /**
   * Algorithmus fr Signaturerstellung, Konstante fr SHA3_256withRSAandMGF1
   */
  public static final String SIGNATURE_ALGORITHM_RSA_SHA3_256 = "http://www.w3.org/2007/05/xmldsig-more#sha3-256-rsa-MGF1";

  /**
   * Algorithmus fr Signaturerstellung, Konstante fr SHA3_256withRSAandMGF1, JCA/JCE-Identifier
   */
  private static final String SIGNATURE_ALGORITHM_RSA_SHA3_256_JCA_JCE = "SHA3-256withRSAandMGF1";

  /**
   * Algorithmus fr Signaturerstellung, Konstante fr SHA3_384withRSAandMGF1
   */
  public static final String SIGNATURE_ALGORITHM_RSA_SHA3_384 = "http://www.w3.org/2007/05/xmldsig-more#sha3-384-rsa-MGF1";

  /**
   * Algorithmus fr Signaturerstellung, Konstante fr SHA3_384withRSAandMGF1, JCA/JCE-Identifier
   */
  private static final String SIGNATURE_ALGORITHM_RSA_SHA3_384_JCA_JCE = "SHA3-384withRSAandMGF1";

  /**
   * Algorithmus fr Signaturerstellung, Konstante fr SHA3_512withRSAandMGF1
   */
  public static final String SIGNATURE_ALGORITHM_RSA_SHA3_512 = "http://www.w3.org/2007/05/xmldsig-more#sha3-512-rsa-MGF1";

  /**
   * Algorithmus fr Signaturerstellung, Konstante fr SHA3_512withRSAandMGF1, JCA/JCE-Identifier
   */
  private static final String SIGNATURE_ALGORITHM_RSA_SHA3_512_JCA_JCE = "SHA3-512withRSAandMGF1";



  /**
   * Algorithmus fr Hashwerterstellung, Konstante fr SHA1
   */
  public static final String DIGEST_ALGORITHM_SHA1 = "http://www.w3.org/2000/09/xmldsig#sha1";

  /**
   * Algorithmus fr Hashwerterstellung, Konstante fr SHA1, JCA/JCE-Identifier
   */
  private static final String DIGEST_ALGORITHM_SHA1_JCA_JCE = "SHA-1";

  /**
   * Algorithmus fr Hashwerterstellung, Konstante fr SHA256
   */
  public static final String DIGEST_ALGORITHM_SHA256 = "http://www.w3.org/2001/04/xmlenc#sha256";

  /**
   * Algorithmus fr Hashwerterstellung, Konstante fr SHA256, JCA/JCE-Identifier
   */
  private static final String DIGEST_ALGORITHM_SHA256_JCA_JCE = "SHA-256";

  /**
   * Algorithmus fr Hashwerterstellung, Konstante fr SHA512
   */
  public static final String DIGEST_ALGORITHM_SHA512 = "http://www.w3.org/2001/04/xmlenc#sha512";

  /**
   * Algorithmus fr Hashwerterstellung, Konstante fr SHA512, JCA/JCE-Identifier
   */
  private static final String DIGEST_ALGORITHM_SHA512_JCA_JCE = "SHA-512";

  /**
   * Algorithmus fr Hashwerterstellung, Konstante fr RIPEMD160
   */
  public static final String DIGEST_ALGORITHM_RIPEMD160 = "http://www.w3.org/2001/04/xmlenc#ripemd160";

  /**
   * Algorithmus fr Hashwerterstellung, Konstante fr RIPEMD160, JCA/JCE-Identifier
   */
  private static final String DIGEST_ALGORITHM_RIPEMD160_JCA_JCE = "RIPEMD160";

  /**
   * Algorithmus fr Hashwerterstellung, Konstante fr SHA3-256
   */
  public static final String DIGEST_ALGORITHM_SHA3_256 = "http://www.w3.org/2007/05/xmldsig-more#sha3-256";

  /**
   * Algorithmus fr Hashwerterstellung, Konstante fr SHA3-256, JCA/JCE-Identifier
   */
  private static final String DIGEST_ALGORITHM_SHA3_256_JCA_JCE = "SHA3-256";

  /**
   * Algorithmus fr Hashwerterstellung, Konstante fr SHA3-384
   */
  public static final String DIGEST_ALGORITHM_SHA3_384 = "http://www.w3.org/2007/05/xmldsig-more#sha3-384";

  /**
   * Algorithmus fr Hashwerterstellung, Konstante fr SHA3-384, JCA/JCE-Identifier
   */
  private static final String DIGEST_ALGORITHM_SHA3_384_JCA_JCE = "SHA3-384";

  /**
   * Algorithmus fr Hashwerterstellung, Konstante fr SHA3-512
   */
  public static final String DIGEST_ALGORITHM_SHA3_512 = "http://www.w3.org/2007/05/xmldsig-more#sha3-512";

  /**
   * Algorithmus fr Hashwerterstellung, Konstante fr SHA3-512, JCA/JCE-Identifier
   */
  private static final String DIGEST_ALGORITHM_SHA3_512_JCA_JCE = "SHA3-512";



  /**
   * Algorithmus fr symmetrische Verschlsselung Two-Key-Triple-DES (CBC-Mode) - Warnung: veraltet!
   * 
   * @deprecated Diese Algorithmen werden in Zukunft nicht mehr zu Verfgung stehen. In Zukunft sollen die
   *             Algorithmen mit GCM Modus benutzt werden
   */
  @Deprecated
  public static final String SYMMETRIC_CIPHER_ALGORITHM_TDES_CBC = "http://www.w3.org/2001/04/xmlenc#tripledes-cbc";

  /**
   * Algorithmus fr symmetrische Verschlsselung Two-Key-Triple-DES (CBC-Mode), JCA/JCE-Identifier - Warnung:
   * veraltet!
   * 
   * @deprecated Diese Algorithmen werden in Zukunft nicht mehr zu Verfgung stehen. In Zukunft sollen die
   *             Algorithmen mit GCM Modus benutzt werden
   */
  @Deprecated
  private static final String SYMMETRIC_CIPHER_ALGORITHM_TDES_CBC_JCA_JCE = "DESede/CBC/PKCS5Padding";

  /**
   * Algorithmus fr symmetrische Verschlsselung AES-128-CBC - Warnung: veraltet!
   * 
   * @deprecated Diese Algorithmen werden in Zukunft nicht mehr zu Verfgung stehen. In Zukunft sollen die
   *             Algorithmen mit GCM Modus benutzt werden
   */
  @Deprecated
  public static final String SYMMETRIC_CIPHER_ALGORITHM_AES128 = "http://www.w3.org/2001/04/xmlenc#aes128-cbc";

  /**
   * Algorithmus fr symmetrische Verschlsselung AES (CBC-Mode), JCA/JCE-Identifier - Warnung: veraltet!
   * 
   * @deprecated Diese Algorithmen werden in Zukunft nicht mehr zu Verfgung stehen. In Zukunft sollen die
   *             Algorithmen mit GCM Modus benutzt werden
   */
  @Deprecated
  private static final String SYMMETRIC_CIPHER_ALGORITHM_AES_CBC_JCA_JCE = "AES/CBC/PKCS5Padding";

  /**
   * Algorithmus fr symmetrische Verschlsselung AES-192-CBC - Warnung: veraltet!
   * 
   * @deprecated Diese Algorithmen werden in Zukunft nicht mehr zu Verfgung stehen. In Zukunft sollen die
   *             Algorithmen mit GCM Modus benutzt werden
   */
  @Deprecated
  public static final String SYMMETRIC_CIPHER_ALGORITHM_AES192 = "http://www.w3.org/2001/04/xmlenc#aes192-cbc";

  /**
   * Algorithmus fr symmetrische Verschlsselung AES-256-CBC - Warnung: veraltet!
   * 
   * @deprecated Diese Algorithmen werden in Zukunft nicht mehr zu Verfgung stehen. In Zukunft sollen die
   *             Algorithmen mit GCM Modus benutzt werden
   */
  @Deprecated
  public static final String SYMMETRIC_CIPHER_ALGORITHM_AES256 = "http://www.w3.org/2001/04/xmlenc#aes256-cbc";

  /**
   * Algorithmus fr symmetrische Verschlsselung AES-128-GCM
   * 
   * @since 1.8.0
   */
  public static final String SYMMETRIC_CIPHER_ALGORITHM_AES128_GCM = "http://www.w3.org/2009/xmlenc11#aes128-gcm";

  /**
   * Standartwert fr den symmetrische Verschlsselung Algorithmus
   */
  public static final String DEFAULT_SYMMETRIC_CIPHER_ALGORITHM = Constants.SYMMETRIC_CIPHER_ALGORITHM_AES256;

  /**
   * Algorithmus fr symmetrische Verschlsselung AES (GCM-Mode), JCA/JCE-Identifier
   * 
   * @since 1.8.0
   */
  private static final String SYMMETRIC_CIPHER_ALGORITHM_AES_GCM_JCA_JCE = "AES/GCM/NoPadding";

  /**
   * Algorithmus fr symmetrische Verschlsselung AES-192-GCM
   * 
   * @since 1.8.0
   */
  public static final String SYMMETRIC_CIPHER_ALGORITHM_AES192_GCM = "http://www.w3.org/2009/xmlenc11#aes192-gcm";

  /**
   * Algorithmus fr symmetrische Verschlsselung AES-256-GCM
   * 
   * @since 1.8.0
   */
  public static final String SYMMETRIC_CIPHER_ALGORITHM_AES256_GCM = "http://www.w3.org/2009/xmlenc11#aes256-gcm";

  /**
   * Algorithmus fr asymmetrische Verschlsselung RSAES-PKCS1-v1_5
   */
  public static final String ASYMMETRIC_CIPHER_ALGORITHM_RSA_1_5 = "http://www.w3.org/2001/04/xmlenc#rsa-1_5";

  /**
   * Algorithmus fr asymmetrische Verschlsselung RSA-OAEP
   */
  public static final String ASYMMETRIC_CIPHER_ALGORITHM_RSA_OAEP = "http://www.w3.org/2009/xmlenc11#rsa-oaep";

  /**
   * Algorithmus fr Maskengenerierung (RSA-OAEP) mit MGF1 und SHA-256
   */
  public static final String MASK_GENERATION_FUNCTION_1_SHA256 = "http://www.w3.org/2009/xmlenc11#mgf1sha256";

  /**
   * Algorithmus fr Maskengenerierung (RSA-OAEP) mit MGF1 und SHA-512
   */
  public static final String MASK_GENERATION_FUNCTION_1_SHA512 = "http://www.w3.org/2009/xmlenc11#mgf1sha512";

  /**
   * Algorithmus fr asymmetrische Verschlsselung RSAES-PKCS1-v1_5, JCA/JCE-Identifier
   */
  private static final String ASYMMETRIC_CIPHER_ALGORITHM_RSA_1_5_JCA_JCE = "RSA/ECB/PKCS1Padding";

  /**
   * Algorithmus fr asymmetrische Verschlsselung RSAES-OAEP, JCA/JCE-Identifier
   */
  private static final String ASYMMETRIC_CIPHER_ALGORITHM_RSA_OAEP_JCA_JCE = "RSA/ECB/OAEPPadding";

  /**
   * Event-Id 'Verbindung zum Intermedir wird aufgebaut'
   */
  public static final int EVENT_CONNECT = 0;

  /**
   * Event-Id 'Auftragsnachricht wird signiert'
   */
  public static final int EVENT_SIGN_MSG = 1;

  /**
   * Event-Id 'Auftragsnachricht wird versendet'
   */
  public static final int EVENT_SEND_MSG = 2;

  /**
   * Event-Id 'Antwortnachricht wird empfangen'
   */
  public static final int EVENT_RECEIVE_MSG = 3;

  /**
   * Event-Id 'Vorgang abgeschlossen'
   */
  public static final int EVENT_ACTION_COMPLETE = 4;

  // Transforms
  /**
   * XSLT-Transformation
   */
  public static final String TRANSFORM_XSLT = "http://www.w3.org/TR/1999/REC-xslt-19991116";

  /**
   * Kanonisierung
   */
  public static final String TRANSFORM_CANONICALIZATION = "http://www.w3.org/TR/2001/REC-xml-c14n-20010315";

  /**
   * Tranformer fr Base64-Decoder
   */
  public static final String TRANSFORM_BASE64 = "http://www.w3.org/2000/09/xmldsig#base64";

  static
  {

    Hashtable<String, String> tmpJcaJce = new Hashtable<String, String>();
    tmpJcaJce.put(ASYMMETRIC_CIPHER_ALGORITHM_RSA_1_5, ASYMMETRIC_CIPHER_ALGORITHM_RSA_1_5_JCA_JCE);
    tmpJcaJce.put(ASYMMETRIC_CIPHER_ALGORITHM_RSA_OAEP, ASYMMETRIC_CIPHER_ALGORITHM_RSA_OAEP_JCA_JCE);

    tmpJcaJce.put(DIGEST_ALGORITHM_SHA1, DIGEST_ALGORITHM_SHA1_JCA_JCE);
    tmpJcaJce.put(DIGEST_ALGORITHM_SHA256, DIGEST_ALGORITHM_SHA256_JCA_JCE);
    tmpJcaJce.put(DIGEST_ALGORITHM_SHA512, DIGEST_ALGORITHM_SHA512_JCA_JCE);
    tmpJcaJce.put(DIGEST_ALGORITHM_RIPEMD160, DIGEST_ALGORITHM_RIPEMD160_JCA_JCE);
    tmpJcaJce.put(DIGEST_ALGORITHM_SHA3_256, DIGEST_ALGORITHM_SHA3_256_JCA_JCE);
    tmpJcaJce.put(DIGEST_ALGORITHM_SHA3_384, DIGEST_ALGORITHM_SHA3_384_JCA_JCE);
    tmpJcaJce.put(DIGEST_ALGORITHM_SHA3_512, DIGEST_ALGORITHM_SHA3_512_JCA_JCE);

    tmpJcaJce.put(SIGNATURE_ALGORITHM_RSA_SHA1, SIGNATURE_ALGORITHM_RSA_SHA1_JCA_JCE);
    tmpJcaJce.put(SIGNATURE_ALGORITHM_RSA_SHA256, SIGNATURE_ALGORITHM_RSA_SHA256_JCA_JCE);
    tmpJcaJce.put(SIGNATURE_ALGORITHM_RSA_SHA512, SIGNATURE_ALGORITHM_RSA_SHA512_JCA_JCE);
    tmpJcaJce.put(SIGNATURE_ALGORITHM_RSA_SHA256_PSS, SIGNATURE_ALGORITHM_RSA_SHA256_PSS_JCA_JCE);
    tmpJcaJce.put(SIGNATURE_ALGORITHM_RSA_SHA512_PSS, SIGNATURE_ALGORITHM_RSA_SHA512_PSS_JCA_JCE);
    tmpJcaJce.put(SIGNATURE_ALGORITHM_RSA_RIPEMD160, SIGNATURE_ALGORITHM_RSA_RIPEMD160_JCA_JCE);
    tmpJcaJce.put(SIGNATURE_ALGORITHM_RSA_RIPEMD160_RFC4051, SIGNATURE_ALGORITHM_RSA_RIPEMD160_JCA_JCE);
    tmpJcaJce.put(SIGNATURE_ALGORITHM_ECDSA_SHA256, SIGNATURE_ALGORITHM_ECDSA_SHA256_JCA_JCE);
    tmpJcaJce.put(SIGNATURE_ALGORITHM_ECDSA_SHA512, SIGNATURE_ALGORITHM_ECDSA_SHA512_JCA_JCE);
    tmpJcaJce.put(SIGNATURE_ALGORITHM_RSA_SHA3_256, SIGNATURE_ALGORITHM_RSA_SHA3_256_JCA_JCE);
    tmpJcaJce.put(SIGNATURE_ALGORITHM_RSA_SHA3_384, SIGNATURE_ALGORITHM_RSA_SHA3_384_JCA_JCE);
    tmpJcaJce.put(SIGNATURE_ALGORITHM_RSA_SHA3_512, SIGNATURE_ALGORITHM_RSA_SHA3_512_JCA_JCE);

    tmpJcaJce.put(SYMMETRIC_CIPHER_ALGORITHM_TDES_CBC, SYMMETRIC_CIPHER_ALGORITHM_TDES_CBC_JCA_JCE);
    tmpJcaJce.put(SYMMETRIC_CIPHER_ALGORITHM_AES128, SYMMETRIC_CIPHER_ALGORITHM_AES_CBC_JCA_JCE);
    tmpJcaJce.put(SYMMETRIC_CIPHER_ALGORITHM_AES192, SYMMETRIC_CIPHER_ALGORITHM_AES_CBC_JCA_JCE);
    tmpJcaJce.put(SYMMETRIC_CIPHER_ALGORITHM_AES256, SYMMETRIC_CIPHER_ALGORITHM_AES_CBC_JCA_JCE);

    tmpJcaJce.put(SYMMETRIC_CIPHER_ALGORITHM_AES128_GCM, SYMMETRIC_CIPHER_ALGORITHM_AES_GCM_JCA_JCE);
    tmpJcaJce.put(SYMMETRIC_CIPHER_ALGORITHM_AES192_GCM, SYMMETRIC_CIPHER_ALGORITHM_AES_GCM_JCA_JCE);
    tmpJcaJce.put(SYMMETRIC_CIPHER_ALGORITHM_AES256_GCM, SYMMETRIC_CIPHER_ALGORITHM_AES_GCM_JCA_JCE);

    JCA_JCE_MAP = new UnmodifiableHashtable<String, String>(tmpJcaJce);

    // Note: (Calendar month based on 0)
    Calendar c = new GregorianCalendar(TimeZone.getTimeZone("Europe/Berlin"));
    c.clear();
    c.set(2014, 2, 1);
    ACTUAL_DATE = c.getTime();

    c.clear();
    c.set(2008, 3, 1);
    OUT_DATE_KEYSIZE_1024 = c.getTime();

    Hashtable<String, Date> tmpOutDates = new Hashtable<String, Date>();

    c.clear();
    c.set(2008, 6, 1); // BSI-Empfehlung
    tmpOutDates.put(DIGEST_ALGORITHM_SHA1, c.getTime());
    tmpOutDates.put(SIGNATURE_ALGORITHM_RSA_SHA1, c.getTime());

    c.clear();
    c.set(2011, 0, 1);
    tmpOutDates.put(DIGEST_ALGORITHM_RIPEMD160, c.getTime());
    tmpOutDates.put(SIGNATURE_ALGORITHM_RSA_RIPEMD160, c.getTime());
    tmpOutDates.put(SIGNATURE_ALGORITHM_RSA_RIPEMD160_RFC4051, c.getTime());

    OUT_DATES = new UnmodifiableHashtable<String, Date>(tmpOutDates);

  }

  // helper class to make some final static Hashtable constants unmodifiable
  private static class UnmodifiableHashtable<K extends Object, V extends Object> extends Hashtable<K, V>
  {

    /**
     *
     */
    private static final long serialVersionUID = 1L;

    private final boolean initialized;

    /**
     * Constructor.
     *
     * @param m the map whose mappings are to be placed in this table.
     */
    public UnmodifiableHashtable(Map<? extends K, ? extends V> m)
    {
      super(m);
      this.initialized = true;
    }

    @Override
    public synchronized V put(K key, V value)
    {
      if (this.initialized)
      {
        throw new UnsupportedOperationException("modifications not supported");
      }
      return super.put(key, value);
    }

    @Override
    public synchronized void putAll(Map<? extends K, ? extends V> t)
    {
      if (this.initialized)
      {
        throw new UnsupportedOperationException("modifications not supported");
      }
      super.putAll(t);
    }

    @Override
    public synchronized V remove(Object key)
    {
      if (this.initialized)
      {
        throw new UnsupportedOperationException("modifications not supported");
      }
      return super.remove(key);
    }

    @Override
    public synchronized void clear()
    {
      if (this.initialized)
      {
        throw new UnsupportedOperationException("modifications not supported");
      }
      super.clear();
    }

  }
}
