package de.osci.osci12.common;

import de.osci.osci12.OSCIException;
import de.osci.osci12.common.OSCIExceptionCodes.OSCIExceptionCodesI;
import de.osci.osci12.messagetypes.OSCIMessage;


/**
 * Diese Exception wird fr die in der OSCI 1.2 Transport-Spezifikation definierten Fehlermeldungen auf
 * Auftragsebene verwendet.
 * <p>
 * Copyright  2017 Freie Hansestadt Bremen
 * </p>
 * <p>
 * Erstellt von Governikus GmbH & Co.  KG
 * </p>
 * <p>
 * Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr freie Softwarebibliotheken genutzt
 * werden.
 * </p>
 * Die Lizenzbestimmungen knnen unter der URL <a href='www.osci.de' >www.osci.de</a > abgerufen oder bei der Senator fr
 * Finanzen, Freie Hansestadt Bremen,  Schillerstr. 1, 28195 Bremen schriftlich angefordert werden.
 * </p>
 *
 * @author R. Lindemann, N.Bngener
 * @version 1.8.0
 */
public class OSCIErrorException extends OSCIException
{

  private static final long serialVersionUID = 1L;

  private OSCIMessage message;

  /**
   * Legt ein neues OSCIErrorException Objekt an.
   *
   * @deprecated Bitte den Konstruktor mit OSCIExceptionCodesI benutzen
   * @param errorCode Fehlercode
   */
  @Deprecated
  public OSCIErrorException(String errorCode)
  {
    super(errorCode);
  }

  /**
   * Legt ein neues OSCIErrorException Objekt an.
   *
   * @deprecated Bitte den Konstruktor mit OSCIExceptionCodesI benutzen
   * @param errorCode Fehlercode
   * @param message OSCI-Nachrichtenobjekt, dessen Verarbeitung den Fehler ausgelst hat.
   */
  @Deprecated
  public OSCIErrorException(String errorCode, OSCIMessage message)
  {
    super(errorCode);
    this.message = message;
  }

  /**
   * Legt ein neues OSCIErrorException Objekt an.
   *
   * @param errorCode Fehlercode Objekt
   */
  public OSCIErrorException(OSCIExceptionCodesI errorCode)
  {
    super(errorCode);
  }

  /**
   * Legt ein neues OSCIErrorException Objekt an.
   *
   * @param errorCode Fehlercode Objekt
   * @param message OSCI-Nachrichtenobjekt, dessen Verarbeitung den Fehler ausgelst hat.
   */
  public OSCIErrorException(OSCIExceptionCodesI errorCode, OSCIMessage message)
  {
    super(errorCode);
    this.message = message;
  }

  /**
   * Liefert das OSCI-Nachrichtenobjekt, dessen Verarbeitung den Fehler ausgelst hat. Wenn beim Auftreten des
   * Fehlers noch kein Nachrichtenobjekt angelegt werden konnte, wir null zurckgegeben. Das Objekt ist in dem
   * Zustand zum Zeitpunkt des Fehlers, also ggf. unvollstndig.
   * 
   * @return OSCI-Nachrichtenobjekt, ggf. null
   */
  public OSCIMessage getOSCIMessage()
  {
    return message;
  }

}
