package de.osci.osci12.encryption;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import de.osci.osci12.common.Constants;
import de.osci.osci12.common.DialogHandler;
import de.osci.osci12.extinterfaces.OSCIDataSource;


/**
 * <p>Copyright  2017 Freie Hansestadt Bremen</p>
 * <p>Erstellt von Governikus GmbH & Co.  KG
 * </p>
 * <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
 * freie Softwarebibliotheken genutzt werden.</p>Die Lizenzbestimmungen knnen
 * unter der URL <a href='www.osci.de' >www.osci.de</a > abgerufen oder bei der
 * Koordinierungsstelle fr IT-Standards (KoSIT),  Schillerstr. 1,
 * 28195 Bremen schriftlich angefordert werden.</p>
 *
 *
 * @author R. Lindemann, N.Bngener
 * @version 1.8.0
 */
public class CipherReference
{
  // Transform ist optinal und wird zunchst nicht untersttzt.
  private static Log log = LogFactory.getLog(CipherReference.class);
  private Vector<String> transformList = new Vector<String>();
  private String uri;
  private OSCIDataSource swapBuffer = null;
  private boolean encryptedStream = false;

  /**
   * Creates a new CipherReference object.
   *
   * @param uri undocumented
   *
   * @throws IOException undocumented
   */
  public CipherReference(String uri) throws IOException
  {
    this.uri = uri;

    if (log.isDebugEnabled())
      log.debug("Konstruktor");

    swapBuffer = DialogHandler.getNewDataBuffer();
  }

  /**
   * undocumented
   *
   * @param refStream undocumented
   *
   * @throws IOException undocumented
   */
  public void setReferencedStream(InputStream refStream)
                           throws IOException
  {
    if (log.isDebugEnabled())
      log.debug("Der Referenced Stream wurde gesetzt.");

    OutputStream out = swapBuffer.getOutputStream();
    byte[] bytes = new byte[1024];
    int anz = 0;

    while ((anz = refStream.read(bytes)) > -1)
    {
      out.write(bytes, 0, anz);
    }
  }

  /**
   * undocumented
   *
   * @param transform undocumented
   */
  public void addTransform(String transform)
  {
    transformList.add(transform);
  }

  /**
   * undocumented
   *
   * @param i undocumented
   *
   * @return undocumented
   */
  public String getTransform(int i)
  {
    return transformList.get(i);
  }

  /**
   * undocumented
   *
   * @param uri undocumented
   */
  public void setURI(String uri)
  {
    this.uri = uri;
  }

  /**
   * undocumented
   *
   * @return undocumented
   */
  public String getURI()
  {
    return uri;
  }

  /**
   * undocumented
   *
   * @return undocumented
   *
   * @throws IOException undocumented
   */
  public InputStream getReferencedStream() throws IOException
  {
    return swapBuffer.getInputStream();
  }

  /**
   * undocumented
   *
   * @param out undocumented
   *
   * @throws IOException undocumented
   */
  public void writeXML(OutputStream out, String ds, String xenc)
                throws IOException
  {
    //id
    out.write(("<" + xenc + ":CipherReference URI=\"" + this.getURI() + "\">").getBytes(Constants.CHAR_ENCODING));
    out.write(("<" + xenc + ":Transforms><" + ds +
              ":Transform Algorithm=\"http://www.w3.org/2000/09/xmldsig#base64\"></" + ds + ":Transform></" + xenc +
              ":Transforms></" + xenc).getBytes(Constants.CHAR_ENCODING));
    // Transformer auflisten
    out.write(":CipherReference>".getBytes(Constants.CHAR_ENCODING));
  }

  /**
   * undocumented
   *
   * @return undocumented
   */
  public boolean isEncryptedStream()
  {
    return encryptedStream;
  }

  /**
   * undocumented
   *
   * @param encryptedStream undocumented
   */
  public void setEncryptedStream(boolean encryptedStream)
  {
    this.encryptedStream = encryptedStream;
  }
}
