package de.osci.osci12.encryption;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;

import javax.crypto.SecretKey;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import de.osci.helper.Base64InputStream;
import de.osci.helper.Base64OutputStream;
import de.osci.helper.SymCipherOutputStream;
import de.osci.osci12.OSCIException;
import de.osci.osci12.common.Constants;
import de.osci.osci12.common.DialogHandler;
import de.osci.osci12.extinterfaces.OSCIDataSource;
import de.osci.osci12.messageparts.ContentContainer;
import de.osci.osci12.messageparts.MessagePartsFactory;


/**
 * <p>Copyright  2017 Freie Hansestadt Bremen</p>
 * <p>Erstellt von Governikus GmbH & Co.  KG
 * </p>
 * <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
 * freie Softwarebibliotheken genutzt werden.</p>Die Lizenzbestimmungen knnen
 * unter der URL <a href='www.osci.de' >www.osci.de</a > abgerufen oder bei der
 * Koordinierungsstelle fr IT-Standards (KoSIT),  Schillerstr. 1,
 * 28195 Bremen schriftlich angefordert werden.</p>
 *
 *
 * @author R. Lindemann, N.Bngener
 * @version 1.8.0
 */
public class CipherValue
{
  private static Log log = LogFactory.getLog(CipherValue.class);
  private OSCIDataSource swapBuffer = null;
  private ContentContainer coco = null;
  private javax.crypto.SecretKey key = null;
  private byte[] iv;
  private String symAlgorithm;

  /**
   * Creates a new CipherValue object.
   *
   * @param data undocumented
   *
   * @throws IOException undocumented
   */
  public CipherValue(String data) throws IOException
  {
    swapBuffer = DialogHandler.getNewDataBuffer();

    if (log.isDebugEnabled())
      log.debug("Konstruktor String Data");

    OutputStream out = swapBuffer.getOutputStream();
    out.write(data.getBytes(Constants.CHAR_ENCODING));
    out.close();
  }

  /**
   * Creates a new CipherValue object.
   *
   * @param dataBytes undocumented
   *
   * @throws IOException undocumented
   */
  public CipherValue(byte[] dataBytes) throws IOException
  {
    swapBuffer = DialogHandler.getNewDataBuffer();

    Base64OutputStream b64out = new Base64OutputStream(swapBuffer.getOutputStream(), false);
    b64out.write(dataBytes);
    b64out.close();
  }

  /**
   * Creates a new CipherValue object.
   *
   * @throws IOException undocumented
   */
  public CipherValue() throws IOException
  {
    swapBuffer = DialogHandler.getNewDataBuffer();
  }

  /**
   * undocumented
   *
   * @return undocumented
   *
   * @throws IOException undocumented
   */
  public OutputStreamWriter getStreamToWrite() throws IOException
  {
    return new OutputStreamWriter(swapBuffer.getOutputStream(), Constants.CHAR_ENCODING);
  }

  /**
   * Creates a new CipherValue object.
   *
   * @param contentContainer undocumented
   * @param key undocumented
   */
  public CipherValue(de.osci.osci12.messageparts.ContentContainer contentContainer, SecretKey key,String symAlgorithm)
  {
    this.coco = contentContainer;
    this.key = key;
    this.symAlgorithm=symAlgorithm;
  }

  /**
   * undocumented
   *
   * @return undocumented
   *
   * @throws IOException undocumented
   */
  public InputStream getCipherValueStream() throws IOException
  {
    swapBuffer.getInputStream().reset();

    return new Base64InputStream(swapBuffer.getInputStream());
  }

  /**
   * undocumented
   *
   * @param out undocumented
   *
   * @throws IOException undocumented
   * @throws OSCIException undocumented
   */
  public void writeXML(OutputStream out, String xenc) throws IOException, 
                                                             OSCIException
  {
    out.write(("<" + xenc + ":CipherValue>").getBytes(Constants.CHAR_ENCODING));

    if (coco != null)
    {
      Base64OutputStream b64out = new Base64OutputStream(out, false);
      SymCipherOutputStream scos = new SymCipherOutputStream(b64out, key,symAlgorithm, iv);
      iv = scos.getIv();
      MessagePartsFactory.writeXML(coco, scos);
      scos.close();
      b64out.flush(true);
    }
    else
    {
      InputStream is = swapBuffer.getInputStream();
      is.reset();

      byte[] tmp = new byte[1024];
      int i;

      while ((i = is.read(tmp)) > -1)
        out.write(tmp, 0, i);

      is.close();
    }

    out.write(("</" + xenc + ":CipherValue>").getBytes(Constants.CHAR_ENCODING));
    out.flush();
  }
}
