package de.osci.osci12.extinterfaces;

import de.osci.osci12.common.DialogHandler;
import de.osci.osci12.soapheader.ControlBlockH;


/**
 * <p>Diese Schnittstelle wird nur noch intern verwendet.</p>
 * <p>Copyright  2017 Freie Hansestadt Bremen</p>
 * <p>Erstellt von Governikus GmbH & Co.  KG
 * </p>
 * <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
 * freie Softwarebibliotheken genutzt werden.</p>Die Lizenzbestimmungen knnen
 * unter der URL <a href='www.osci.de' >www.osci.de</a > abgerufen oder bei der
 * Koordinierungsstelle fr IT-Standards (KoSIT),  Schillerstr. 1,
 * 28195 Bremen schriftlich angefordert werden.</p>
 *
 *
 * @author R. Lindemann / N. Bngener
 * @version 1.8.0
 * @see de.osci.osci12.common.DialogHandler
 */
public abstract class DialogFinder
{
  /**
   * Diese Methode muss anhand des bergebenen ControlBlockH-Objektes das
   * zugehrige DialogHandler-Objekt ermitteln und zurckgeben.
   * @param controlBlock das ControlBlock-Objekt als Identifier des gesuchten
   * DialogHandlers
   * @return gefundener DialogHandler oder null, wenn er nicht gefunden wurde
   */
  public abstract DialogHandler findDialog(ControlBlockH controlBlock);

  /**
   * Entfernt ein DialogHandler-Objekt aus der Liste der verwalteten
   * DialogHandler-Objekte. Wird nach Beendigung eines Dialogs
   * aufgerufen.
   * @param controlBlock das ControlBlock-Objekt als Identifier des gesuchten
   * DialogHandlers
   * @return true, wenn der DialogHandler erfolgreich entfernt wurde
   */
  public abstract boolean removeDialog(ControlBlockH controlBlock);

  /**
   * Fgt den verwalteten DialogHandler-Objekten ein weiteres hinzu.
   * @param dialog
   * @throws Exception
   */
  public abstract void addDialog(DialogHandler dialog)
                          throws Exception;

  /**
   * Liefert eine Versionsnummer.
   * @return Versionsnummer
   */
  public abstract String getVersion();

  /**
   * Liefert den Namen des Herstellers.
   * @return Herstellername
   */
  public abstract String getVendor();
}
