package de.osci.osci12.messageparts;

import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;

import de.osci.osci12.common.DialogHandler;
import de.osci.osci12.messagetypes.OSCIMessage;
import de.osci.osci12.messagetypes.OSCIMessageBuilder;


/**
 * Interne Klasse, wird von Anwendungen nicht direkt bentigt.
 *
 * <p>Copyright  2017 Freie Hansestadt Bremen</p>
 * <p>Erstellt von Governikus GmbH & Co.  KG
 * </p>
 * <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
 * freie Softwarebibliotheken genutzt werden.</p>Die Lizenzbestimmungen knnen
 * unter der URL <a href='www.osci.de' >www.osci.de</a > abgerufen oder bei der
 * Koordinierungsstelle fr IT-Standards (KoSIT),  Schillerstr. 1,
 * 28195 Bremen schriftlich angefordert werden.</p>
 *
 *
 * @author R. Lindemann, N. Bngener
 * @version 1.8.0
 */
public class MessagePartParser extends DefaultHandler
{
  //  private static Log log = LogFactory.getLog(MessagePartParser.class);
  protected DefaultHandler parentHandler = null;
  protected XMLReader xmlReader = null;
  protected StringBuffer currentElement = null;
  protected OSCIMessage msg = null;
  protected static final String SOAP_XMLNS = "http://schemas.xmlsoap.org/soap/envelope/";
  protected static final String OSCI_XMLNS = "http://<a href='www.osci.de' >www.osci.de</a >/2002/04/osci";
  protected static final String DS_XMLNS = "http://www.w3.org/2000/09/xmldsig#";
  protected static final String XENC_XMLNS = "http://www.w3.org/2001/04/xmlenc#";
  protected static final String XSI_XMLNS = "http://www.w3.org/2001/XMLSchema-instance";
  protected static final String XADES_XMLNS = "http://uri.etsi.org/01903/v1.3.2#";

  /**
   * Creates a new MessagePartParser object.
   *
   * @param parentHandler undocumented
   */
  protected MessagePartParser(OSCIMessageBuilder parentHandler)
  {
    this.parentHandler = parentHandler;
    msg = parentHandler.getOSCIMessage();
    this.xmlReader = parentHandler.getXMLReader();
  }

  /**
   * Creates a new MessagePartParser object.
   *
   * @param xmlReader undocumented
   * @param parentHandler undocumented
   */
  protected MessagePartParser(XMLReader xmlReader, DefaultHandler parentHandler)
  {
    this.parentHandler = parentHandler;
    this.xmlReader = xmlReader;
  }

  /**
   * undocumented
   *
   * @param ch undocumented
   * @param start undocumented
   * @param length undocumented
   *
   * @throws SAXException undocumented
   */
  public void characters(char[] ch, int start, int length)
                  throws SAXException
  {
    if (currentElement == null)
    {
      for (int i = 0; i < length; i++)
      {
        if (ch[start + i] > ' ')
          throw new SAXException(DialogHandler.text.getString("unexpected_char"));
      }
    }
    else
      currentElement.append(ch, start, length);
  }
}
