/* Generated by Together */
package de.osci.osci12.messagetypes;

import java.io.IOException;
import java.security.NoSuchAlgorithmException;

import de.osci.helper.Tools;
import de.osci.osci12.OSCIException;
import de.osci.osci12.common.Constants.LanguageTextEntries;
import de.osci.osci12.common.DialogHandler;
import de.osci.osci12.messageparts.Body;
import de.osci.osci12.roles.OSCIRoleException;
import de.osci.osci12.roles.Originator;


/**
 * <p>Diese Klasse dient der Beendigung eines expliziten Dialogs.
 * Clients erhalten als Antwort auf diese Nachricht vom Intermedir ein
 * Nachrichtenobjekt, welches eine Rckmeldung ber den Erfolg der Operation
 * enthlt. Diese Rckmeldung wird von der send()-Methode ausgewertet, so
 * dass eine Behandlung der Anwortnachricht selbst nicht erforderlich ist.</p>
 * <p>Copyright  2017 Freie Hansestadt Bremen</p>
 * <p>Erstellt von Governikus GmbH & Co.  KG
 * </p>
 * <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
 * freie Softwarebibliotheken genutzt werden.</p>Die Lizenzbestimmungen knnen
 * unter der URL <a href='www.osci.de' >www.osci.de</a > abgerufen oder bei der
 * Koordinierungsstelle fr IT-Standards (KoSIT),  Schillerstr. 1,
 * 28195 Bremen schriftlich angefordert werden.</p>
 *
 *
 * @author R. Lindemann, N. Bngener
 * @version 1.8.0
 * @see de.osci.osci12.common.DialogHandler
 */
public class ExitDialog extends OSCIRequest
{
  //  private static Log log = LogFactory.getLog(ExitDialog.class);
  /**
   * Legt ein Nachrichtenobjekt zur Dialogbeendigung an.
   *
   * @param dh DialogHandler-Objekt, welches den expliziten Dialog reprsentiert
   *
   * @see de.osci.osci12.common.DialogHandler
   */
  public ExitDialog(DialogHandler dh) throws NoSuchAlgorithmException
  {
    super(dh);
    messageType = EXIT_DIALOG;
    originator = ((Originator) dh.getClient());
    dialogHandler.getControlblock().setResponse(dialogHandler.prevChallenge);
    dialogHandler.getControlblock().setChallenge(Tools.createRandom(10));
    dialogHandler.getControlblock().setSequenceNumber(dialogHandler.getControlblock().getSequenceNumber() + 1);
  }

  ExitDialog()
  {
    super();
    messageType = EXIT_DIALOG;
  }

  /**
   * Versendet die Nachricht und liefert die Antwortnachricht zurck.
   * Diese Methode wirft eine Exception, wenn beim Aufbau oder Versand der
   * Nachricht ein Fehler auftritt. Fehlermeldungen vom Intermedir mssen
   * dem Feedback der Antwortnachricht entnommen werden.
   *
   * @throws IOException bei Ein-/Ausgabefehlern
   * @throws de.osci.osci12.OSCIException bei OSCI-Fehlern
   * @return Antwortnachricht-Objekt
   */
  public ResponseToExitDialog send() throws IOException, 
                                            de.osci.osci12.OSCIException, 
                                            NoSuchAlgorithmException
  {
    return (ResponseToExitDialog) transmit(null, null);
  }

  /**
   * undocumented
   *
   * @throws OSCIRoleException undocumented
   * @throws IllegalStateException undocumented
   */
  protected void compose() throws OSCIException, 
                                  NoSuchAlgorithmException, 
                                  IOException
  {
    super.compose();
    if(featureDescription!=null && dialogHandler.isSendFeatureDescription())
    {
      messageParts.add(featureDescription);
    }
    messageParts.addAll(customHeaders);

    if (dialogHandler.getControlblock().getChallenge() == null)
      throw new IllegalStateException(DialogHandler.text.getString(LanguageTextEntries.missing_entry.name()) + ": Challenge");

    if (dialogHandler.getControlblock().getResponse() == null)
      throw new IllegalStateException(DialogHandler.text.getString(LanguageTextEntries.missing_entry.name()) + ": Response");

    if (dialogHandler.getControlblock().getConversationID() == null)
      throw new IllegalStateException(DialogHandler.text.getString(LanguageTextEntries.missing_entry.name()) + ": Conversation-Id");

    if (dialogHandler.getControlblock().getSequenceNumber() == -1)
      throw new IllegalStateException(DialogHandler.text.getString(LanguageTextEntries.missing_entry.name()) + ": SequenceNumber");

    body = new Body("<" + osciNSPrefix + ":exitDialog></" + osciNSPrefix + ":exitDialog>");
    body.setNSPrefixes(this);
    messageParts.add(body);
    stateOfMsg |= STATE_COMPOSED;
  }
}
