package de.osci.osci12.messagetypes;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.xml.sax.XMLReader;

import de.osci.osci12.OSCIException;
import de.osci.osci12.common.DialogHandler;
import de.osci.osci12.common.OSCIErrorException;
import de.osci.osci12.common.OSCIExceptionCodes.OSCIErrorCodes;
import de.osci.osci12.signature.OSCISignatureException;


/**
 * Die Klasse ist der Eintrittspunkt fr Nachrichten, die bei einem passiven Empfnger
 * eingehen.
 *
 * <p>Copyright  2017 Freie Hansestadt Bremen</p>
 * <p>Erstellt von Governikus GmbH & Co.  KG
 * </p>
 * <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
 * freie Softwarebibliotheken genutzt werden.</p>Die Lizenzbestimmungen knnen
 * unter der URL <a href='www.osci.de' >www.osci.de</a > abgerufen oder bei der
 * Koordinierungsstelle fr IT-Standards (KoSIT),  Schillerstr. 1,
 * 28195 Bremen schriftlich angefordert werden.</p>
 *
 *
 * @author R. Lindemann, N.Bngener
 * @version 1.8.0
 */
public class PassiveRecipientParser extends de.osci.osci12.messagetypes.IncomingMSGParser
{
  private static Log log = LogFactory.getLog(PassiveRecipientParser.class);
  OSCIEnvelopeBuilder getParser(XMLReader reader, DialogHandler dh)
  {
    return new OSCIEnvelopeBuilder(reader, dh);
  }

  /**
   * Diese Methode parst eine Nachricht, die ber einen InputStream eingeht.
   * Sie wird in der Regel von einem Servlet (o..) aufgerufen.
   *
   * @param input InputStream der eingehenden Daten
   * @return eingelesene OSCI-Auftragsnachricht (Annahme- oder Bearbeitungsauftrag)
   * @throws IOException bei Leseproblemen
   * @throws OSCIException bei OSCI-Fehlern
   */
  public OSCIRequest parseStream(InputStream input)
                          throws IOException, 
                                 OSCIException, 
                                 java.security.NoSuchAlgorithmException
  {
    try
    {
      return this.parseStream(input, null);
    }
    catch (OSCISignatureException ex)
    {
      log.error("Error: ",ex);
      throw new OSCIErrorException(OSCIErrorCodes.SignatureInvalid);
    }
  }

  /**
   * Diese Methode parst eine Nachricht, die ber einen InputStream eingeht.
   * Die eingehenden Daten werden parallel in den bergebenen OutputStream
   * geschrieben.
   *
   * @param input InputStream der eingehenden Daten
   * @param storeStream Sicherungsstream
   * @return eingelesene OSCI-Auftragsnachricht (Annahme- oder Bearbeitungsauftrag)
   * @throws IOException bei Leseproblemen
   * @throws OSCIException bei OSCI-Fehlern
   * @see #parseStream(InputStream)
   */
  public OSCIRequest parseStream(InputStream input, OutputStream storeStream)
                          throws IOException, 
                                 OSCIException, 
                                 java.security.NoSuchAlgorithmException
  {
    try
    {
      defaultSupplier = DialogHandler.getDefaultSuppliers();

      return (OSCIRequest) super.parseStream(input, null, true, storeStream);
    }
    catch (OSCISignatureException ex)
    {
      log.error("Error: ",ex);
      throw new OSCIErrorException(OSCIErrorCodes.SignatureInvalid);
    }
  }
}
