package de.osci.osci12.messagetypes;

import java.io.IOException;
import java.security.NoSuchAlgorithmException;

import de.osci.osci12.OSCIException;
import de.osci.osci12.common.Constants.LanguageTextEntries;
import de.osci.osci12.common.DialogHandler;
import de.osci.osci12.messageparts.Body;
import de.osci.osci12.roles.OSCIRoleException;
import de.osci.osci12.roles.Originator;


/**
 * <p>Diese Klasse reprsentiert die Antwortnachricht auf die Beendigung
 * eines expliziten Dialogs.
 * Clients erhalten vom Intermedir eine Instanz dieser Klasse als Antwort auf
 * ein an den Intermedir gesendetes ExitDialog-Objekt.</p>
 * <p>Copyright  2017 Freie Hansestadt Bremen</p>
 * <p>Erstellt von Governikus GmbH & Co.  KG
 * </p>
 * <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
 * freie Softwarebibliotheken genutzt werden.</p>Die Lizenzbestimmungen knnen
 * unter der URL <a href='www.osci.de' >www.osci.de</a > abgerufen oder bei der
 * Koordinierungsstelle fr IT-Standards (KoSIT),  Schillerstr. 1,
 * 28195 Bremen schriftlich angefordert werden.</p>
 *
 *
 * @author R. Lindemann, N. Bngener
 * @version 1.8.0
 * @see de.osci.osci12.messagetypes.ExitDialog
 * @see de.osci.osci12.common.DialogHandler
 */
public class ResponseToExitDialog extends OSCIResponseTo
{
  //  private static Log log = LogFactory.getLog(ResponseToExitDialog.class);
  ResponseToExitDialog(DialogHandler dh)
  {
    super(dh);
    messageType = RESPONSE_TO_EXIT_DIALOG;
    originator = ((Originator) dh.getClient());
    // Challenge wird in compose() zu null gesetzt.
    dialogHandler.getControlblock().setResponse(dialogHandler.prevChallenge);
  }

  ResponseToExitDialog()
  {
    super();
    messageType = RESPONSE_TO_EXIT_DIALOG;
    dialogHandler.explicitDialog = false;
  }

  /**
   * undocumented
   *
   * @throws OSCIRoleException undocumented
   * @throws IllegalStateException undocumented
   */
  protected void compose() throws OSCIException, 
                                  NoSuchAlgorithmException, 
                                  IOException
  {
    super.compose();
    if(featureDescription!=null && dialogHandler.isSendFeatureDescription())
    {
      messageParts.add(featureDescription);
    }
    messageParts.addAll(customHeaders);

    if (dialogHandler.getControlblock().getResponse() == null)
      throw new IllegalStateException(DialogHandler.text.getString(LanguageTextEntries.missing_entry.name()) + ": Response");

    if (dialogHandler.getControlblock().getConversationID() == null)
      throw new IllegalStateException(DialogHandler.text.getString(LanguageTextEntries.missing_entry.name()) + ": Conversation-Id");

    if (dialogHandler.getControlblock().getSequenceNumber() == -1)
      throw new IllegalStateException(DialogHandler.text.getString(LanguageTextEntries.missing_entry.name()) + ": SequenceNumber");

    dialogHandler.getControlblock().setChallenge(null);
    dialogHandler.explicitDialog = false;

    if (feedBack == null)
      throw new IllegalStateException(DialogHandler.text.getString(LanguageTextEntries.missing_entry.name()) + ": Feedback");

    body = new Body("<" + osciNSPrefix + ":responseToExitDialog>" + writeFeedBack() + "</" + osciNSPrefix +
                    ":responseToExitDialog>");
    body.setNSPrefixes(this);
    messageParts.add(body);
    stateOfMsg |= STATE_COMPOSED;
  }
}
