package de.osci.osci12.messagetypes;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

import de.osci.osci12.common.DialogHandler;


/**
 * StoredEnvelope-Parser.
 *
 * <p>Copyright  2017 Freie Hansestadt Bremen</p>
 * <p>Erstellt von Governikus GmbH & Co.  KG
 * </p>
 * <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
 * freie Softwarebibliotheken genutzt werden.</p>Die Lizenzbestimmungen knnen
 * unter der URL <a href='www.osci.de' >www.osci.de</a > abgerufen oder bei der
 * Koordinierungsstelle fr IT-Standards (KoSIT),  Schillerstr. 1,
 * 28195 Bremen schriftlich angefordert werden.</p>
 *
 *
 * @author R. Lindemann / N. Bngener
 * @version 1.8.0
 */
class StoredEnvelopeBuilder extends OSCIEnvelopeBuilder
{
  private static Log log = LogFactory.getLog(StoredEnvelopeBuilder.class);

  /**
   * Creates a new StoredEnvelopeBuilder object.
   *
   * @param xmlReader undocumented
   */
  public StoredEnvelopeBuilder(XMLReader xmlReader)
  {
    super(xmlReader, null);
  }

  /**
   *  Description of the Method
   *
   *@param  uri                           Description of Parameter
   *@param  localName                     Description of Parameter
   *@param  qName                         Description of Parameter
   *@param  attributes                    Description of Parameter
   *@exception  SAXException  Description of Exception
   */
  public void startElement(String uri, String localName, String qName, Attributes attributes)
                    throws SAXException
  {
    if (log.isDebugEnabled())
      log.debug("Start-Element: " + localName);

    // SOAP-Namespacedefinition suchen
    if ((soapNSPrefix == null) || !localName.equals("Envelope"))
      throw new SAXException(DialogHandler.text.getString("msg_format_error") + ": " + localName);

    String schemaLocation = attributes.getValue("xsi:schemaLocation");

    if (log.isDebugEnabled())
      log.debug("Nachrichtentyp: " + schemaLocation);

    if (schemaLocation == null)
    {
      childBuilder = new SOAPFaultBuilder(this);
      this.xmlReader.setContentHandler(childBuilder);
    }
    else if (schemaLocation.equals(XSD_ENCRYPTED_DATA))
    {
      throw new SAXException(new IllegalArgumentException(DialogHandler.text.getString("encrypted_message")));
    }
    else
    {
      int msgType;

      if (schemaLocation.equals(XSD_STORE_DELIVERY))
        msgType = OSCIMessage.STORE_DELIVERY;
      else if (schemaLocation.equals(XSD_RSP_STORE_DELIVERY))
        msgType = OSCIMessage.RESPONSE_TO_STORE_DELIVERY;
      else if (schemaLocation.equals(XSD_FETCH_DELIVERY))
        msgType = OSCIMessage.FETCH_DELIVERY;
      else if (schemaLocation.equals(XSD_RSP_FETCH_DELIVERY))
        msgType = OSCIMessage.RESPONSE_TO_FETCH_DELIVERY;
      else if (schemaLocation.equals(XSD_FORWARD_DELIVERY))
        msgType = OSCIMessage.FORWARD_DELIVERY;
      else if (schemaLocation.equals(XSD_ACCEPT_DELIVERY))
        msgType = OSCIMessage.ACCEPT_DELIVERY;
      else if (schemaLocation.equals(XSD_RSP_ACCEPT_DELIVERY))
        msgType = OSCIMessage.RESPONSE_TO_ACCEPT_DELIVERY;
      else if (schemaLocation.equals(XSD_RSP_FORWARD_DELIVERY))
        msgType = OSCIMessage.RESPONSE_TO_FORWARD_DELIVERY;
      else if (schemaLocation.equals(XSD_MEDIATE_DELIVERY))
        msgType = OSCIMessage.MEDIATE_DELIVERY;
      else if (schemaLocation.equals(XSD_PROCESS_DELIVERY))
        msgType = OSCIMessage.PROCESS_DELIVERY;
      else if (schemaLocation.equals(XSD_RSP_PROCESS_DELIVERY))
        msgType = OSCIMessage.RESPONSE_TO_PROCESS_DELIVERY;
      else if (schemaLocation.equals(XSD_RSP_MEDIATE_DELIVERY))
        msgType = OSCIMessage.RESPONSE_TO_MEDIATE_DELIVERY;
      else if (schemaLocation.equals(XSD_FETCH_PROCESS_CARD))
        msgType = OSCIMessage.FETCH_PROCESS_CARD;
      else if (schemaLocation.equals(XSD_RSP_FETCH_PROCESS_CARD))
        msgType = OSCIMessage.RESPONSE_TO_FETCH_PROCESS_CARD;
      else
      {
        log.error("Nicht erlaubter Nachrichtentyp ! Nachrichtentypt ist:\n" + schemaLocation + "\n" +
                  (XSD_RSP_FETCH_PROCESS_CARD));
        throw new SAXException(DialogHandler.text.getString("sax_exception_msgtype"));
      }

      childBuilder = new StoredMessageBuilder(this, msgType);
      this.xmlReader.setContentHandler(childBuilder);
    }

    childBuilder.getOSCIMessage().soapNSPrefix = soapNSPrefix;
    childBuilder.getOSCIMessage().osciNSPrefix = osciNSPrefix;
    childBuilder.getOSCIMessage().osci2017NSPrefix = osci2017NSPrefix;
    childBuilder.getOSCIMessage().dsNSPrefix = dsNSPrefix;
    childBuilder.getOSCIMessage().xencNSPrefix = xencNSPrefix;
    childBuilder.getOSCIMessage().ns = ns.toString();
  }
}
