package de.osci.osci12.roles;

import java.security.cert.X509Certificate;

import de.osci.osci12.extinterfaces.crypto.Decrypter;
import de.osci.osci12.extinterfaces.crypto.Signer;


/**
 *  Diese Klasse stellt einen OSCI-Autor dar.
 *
 * <p>Copyright  2017 Freie Hansestadt Bremen</p>
 * <p>Erstellt von Governikus GmbH & Co.  KG
 * </p>
 * <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
 * freie Softwarebibliotheken genutzt werden.</p>Die Lizenzbestimmungen knnen
 * unter der URL <a href='www.osci.de' >www.osci.de</a > abgerufen oder bei der
 * Koordinierungsstelle fr IT-Standards (KoSIT),  Schillerstr. 1,
 * 28195 Bremen schriftlich angefordert werden.</p>
 *
 *
 * @author R. Lindemann, N.Bngener
 * @version 1.8.0
 */
public class Author extends Role
{
  private static int idNr = -1;

  /**
   *  Konstruktor fr den Versand einer Nachricht. Wird als zweiter Parameter
   *  null bergeben, weil keine Verschlsselung der Rckantwort gewnscht wird,
   *  muss trotzdem mit setCipherCertificate(X509Certificate) ein
   *  Verschlsselungszertifikat gesetzt werden, weil dies Voraussetzung fr
   *  die Teilnahme an OSCI ist.
   *
   *  @param  signer Signer-Objekt, welches die Signatur der Nachricht erstellen
   *  soll (kann null sein, wenn keine Signatur gewnscht)
   *  @param  decrypter Decrypter-Objekt, welches den Inhalt der Rckantwort entschlsseln
   *  soll (null, wenn die Nachricht nicht verschlsselt wird)
   */
  public Author(Signer signer, Decrypter decrypter)
  {
    super(signer, decrypter);
    idNr++;
    id += idNr;
  }

  /**
   *  Konstruktor fr den Empfang einer Nachricht.
   *
   *  @param  signatureCertificate Zertifikat, mit dem die Signatur der Nachricht geprft wird
   *  @param  cipherCertificate Zertifikat, mit dem die Rckantwort verschlsselt werden soll
   */
  public Author(X509Certificate signatureCertificate, X509Certificate cipherCertificate)
  {
    super(signatureCertificate, cipherCertificate);
    idNr++;
    id += idNr;
  }
}
