package de.osci.osci12.roles;

import java.security.cert.X509Certificate;

import de.osci.osci12.extinterfaces.crypto.Decrypter;
import de.osci.osci12.extinterfaces.crypto.Signer;


/**
 *  Diese Klasse stellt einen OSCI-Intermedir dar.
 *
 * <p>Copyright  2017 Freie Hansestadt Bremen</p>
 * <p>Erstellt von Governikus GmbH & Co.  KG
 * </p>
 * <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
 * freie Softwarebibliotheken genutzt werden.</p>Die Lizenzbestimmungen knnen
 * unter der URL <a href='www.osci.de' >www.osci.de</a > abgerufen oder bei der
 * Koordinierungsstelle fr IT-Standards (KoSIT),  Schillerstr. 1,
 * 28195 Bremen schriftlich angefordert werden.</p>
 *
 *
 * @author R. Lindemann, N.Bngener
 * @version 1.8.0
 */
public class Intermed extends Role
{
  //    private static final String idName = "intermed";
  private java.net.URI uri = null;

  /**
   * Konstruktor fr ein Intermedirs-Objekt (intermedirsseitig).
   *
   * @param signer Signer-Objekt, welches die Signaturen der Nachrichten
   *        erstellen soll (null, wenn keine Signaturen gewnscht).
   * @param decrypter Decrypter-Objekt, welches den Inhalt der Nachrichten
   *        entschlsseln soll.
   */
  public Intermed(Signer signer, Decrypter decrypter)
  {
    super(signer, decrypter);
  }

  /**
   * Konstruktor fr ein Intermedirs-Objekt (clientseitig).
   * Das Signaturzertifikat des Intermedirs muss nicht bergeben werden.
   * Die verwendete URL muss durch die Transportimplementierung aufgelst werden knnen.
   *
   * @param signatureCertificate Zertifikat, mit dem die Signatur einer Antwort
   * geprft werden kann
   * @param cipherCertificate Zertifikat, mit dem die Nachricht verschlsselt
   *        werden soll
   * @param uri URL des Intermedirs.
   */
  public Intermed(X509Certificate signatureCertificate, X509Certificate cipherCertificate, java.net.URI uri)
  {
    super(signatureCertificate, cipherCertificate);
    this.uri = uri;
  }

  /**
   * Liefert die URI des Intermedirs.
   *
   * @return URI
   */
  public java.net.URI getUri()
  {
    return uri;
  }

  /**
   * Setzt die URI des Intermedirs.
   * @param uri die URI
   */
  public void setUri(java.net.URI uri)
  {
    this.uri = uri;
  }
}
