package de.osci.osci12.soapheader;

import java.io.IOException;
import java.io.OutputStream;
import java.security.cert.CertificateEncodingException;
import java.security.cert.X509Certificate;
import java.util.Hashtable;

import de.osci.helper.Base64OutputStream;
import de.osci.osci12.common.Constants;
import de.osci.osci12.common.DialogHandler;
import de.osci.osci12.roles.Addressee;
import de.osci.osci12.roles.Author;
import de.osci.osci12.roles.Intermed;
import de.osci.osci12.roles.OSCIRoleException;
import de.osci.osci12.roles.Originator;
import de.osci.osci12.roles.Reader;
import de.osci.osci12.roles.Role;


/**
 * <p>Copyright  2017 Freie Hansestadt Bremen</p>
 * <p>Erstellt von Governikus GmbH & Co.  KG
 * </p>
 * <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
 * freie Softwarebibliotheken genutzt werden.</p>Die Lizenzbestimmungen knnen
 * unter der URL <a href='www.osci.de' >www.osci.de</a > abgerufen oder bei der
 * Koordinierungsstelle fr IT-Standards (KoSIT),  Schillerstr. 1,
 * 28195 Bremen schriftlich angefordert werden.</p>
 *
 *
 * @author R. Lindemann, N.Bngener
 * @version 1.8.0
 */
public abstract class CertificateH extends HeaderEntry
{
  //  private static Log log = LogFactory.getLog(CertificateH.class);
  protected Hashtable<String, X509Certificate> certificates = new Hashtable<String, X509Certificate>();
  protected byte[] tmpBuffer;

  /**
   * undocumented
   *
   * @param role undocumented
   * @param out undocumented
   *
   * @throws OSCIRoleException undocumented
   * @throws IOException undocumented
   * @throws IllegalArgumentException undocumented
   */
  protected void addCipherCertificate(Role role, OutputStream out)
                               throws OSCIRoleException, 
                                      IOException
  {
    if (role == null)
      throw new IllegalArgumentException(DialogHandler.text.getString("invalid_firstargument") + " null");

    String name = ""; // = role.getClass().getName().substring(role.getClass().getName().lastIndexOf('.') + 1);
    role.getCipherCertificate(); // Test, ob vorhanden

    if (role instanceof Intermed)
      name = "Intermediary";
    else if (role instanceof Originator)
      name = "Originator";
    else if (role instanceof Addressee)
      name = "Addressee";
    else if (role instanceof de.osci.osci12.roles.Reader)
      name = "OtherReader";
    else if (role instanceof Author)
      name = "OtherAuthor";
    else
      throw new IllegalArgumentException(DialogHandler.text.getString("invalid_firstargument") + role);

    /*
       if (name.equals("Reader") || name.equals("Author"))
         name = "Other" + name;
       else if (name.equals("Intermed"))
         name = "Intermediary";
     */
    out.write(("<" + osciNSPrefix + ":CipherCertificate" + name + " Id=\"" + role.getCipherCertificateId() + "\"><" +
              dsNSPrefix + ":X509Data><" + dsNSPrefix + ":X509Certificate>").getBytes(Constants.CHAR_ENCODING));

    try
    {
      Base64OutputStream base64Out = new Base64OutputStream(out, true);
      base64Out.write(role.getCipherCertificate().getEncoded());
      base64Out.flush();
    }
    catch (CertificateEncodingException ex)
    {
      throw new IOException(de.osci.osci12.common.DialogHandler.text.getString("cert_gen_error") + " - " +
                            ex.getClass() + ": " + ex.getMessage());
    }

    out.write(("</" + dsNSPrefix + ":X509Certificate></" + dsNSPrefix + ":X509Data></" + osciNSPrefix +
              ":CipherCertificate" + name + ">").getBytes(Constants.CHAR_ENCODING));
    //    out.flush();
  }

  /**
   * undocumented
   *
   * @return undocumented
   */
  public Hashtable<String, X509Certificate> getCertificates()
  {
    return certificates;
  }

  /**
   * undocumented
   *
   * @param role undocumented
   * @param out undocumented
   *
   * @throws OSCIRoleException undocumented
   * @throws IOException undocumented
   */
  protected void addSignatureCertificate(Role role, OutputStream out)
                                  throws OSCIRoleException, 
                                         IOException
  {
    String name = ""; // = role.getClass().getName().substring(role.getClass().getName().lastIndexOf('.') + 1);
    role.getSignatureCertificate(); // Test, ob vorhanden

    if (role instanceof Intermed)
      name = "Intermediary";
    else if (role instanceof Originator)
      name = "Originator";
    else if (role instanceof Addressee)
      name = "Addressee";
    else if (role instanceof Reader)
      name = "OtherReader";
    else if (role instanceof Author)
      name = "OtherAuthor";
    else
      throw new IllegalArgumentException(DialogHandler.text.getString("invalid_firstargument") + role);

    /*
       if (name.equals("Reader") || name.equals("Author"))
         name = "Other" + name;
       else if (name.equals("Intermed"))
         name = "Intermediary";
     */
    out.write(("<" + osciNSPrefix + ":SignatureCertificate" + name + " Id=\"" + role.getSignatureCertificateId() +
              "\"><" + dsNSPrefix + ":X509Data><" + dsNSPrefix + ":X509Certificate>").getBytes(Constants.CHAR_ENCODING));

    try
    {
      Base64OutputStream base64Out = new Base64OutputStream(out, true);
      base64Out.write(role.getSignatureCertificate().getEncoded());
      base64Out.flush();
    }
    catch (CertificateEncodingException ex)
    {
      throw new IOException(de.osci.osci12.common.DialogHandler.text.getString("cert_gen_error") + " - " +
                            ex.getClass() + ": " + ex.getMessage());
    }

    out.write(("</" + dsNSPrefix + ":X509Certificate></" + dsNSPrefix + ":X509Data></" + osciNSPrefix +
              ":SignatureCertificate" + name + ">").getBytes(Constants.CHAR_ENCODING));
  }
}
